@@ -12,7 +12,7 @@ import sys
 from configparser import ConfigParser
 from datetime import datetime, timedelta
 from pathlib import Path
-import signal
+import yaml
 
 import boto3
 import inquirer
@@ -25,10 +25,14 @@ AWS_CREDENTIAL_PATH = f'{Path.home()}/.aws/credentials'
 AWS_SSO_CACHE_PATH = f'{Path.home()}/.aws/sso/cache'
 AWS_DEFAULT_REGION = 'us-west-1'
 CURRENT_REGION = os.getenv('AWS_REGION')
+AWS_OPS1_ACCOUNT = '101790543458'
+AWS_OPS1_REGION = 'us-west-1'
 
 if not CURRENT_REGION:
     current_session = boto3.session.Session()
     CURRENT_REGION = current_session.region_name
+if not CURRENT_REGION:
+    CURRENT_REGION = 'us-west-1'
 
 default_config = {
   'sso_start_url': 'https://hireteammate.awsapps.com/start',
@@ -60,23 +64,49 @@ def main():
     parser = argparse.ArgumentParser(description='Retrieves AWS credentials from SSO for use with CLI/Boto3 apps.')
     subparsers = parser.add_subparsers()
 
-    parser_sso = subparsers.add_parser('sso')
-    parser_sso_sub = parser_sso.add_subparsers()
-    parser_sso_config = parser_sso_sub.add_parser('config',
-                        help='config aws sso.')
-    parser_sso_config.set_defaults(func=_sso_config)
+    parser.add_argument('-l',
+        action='store_true',
+        help='aws sso login.')
+    parser.add_argument('-c',
+        action='store_true',
+        help='aws sso config.')
+    parser.add_argument('-s',
+        #action='store_true',
+        nargs='*',
+        help='aws sso switch account.')
+    parser.add_argument('-g',
+        action='store_true',
+        help='aws sso logout.')
+#    parser_sso = subparsers.add_parser('sso')
+#    parser_sso_sub = parser_sso.add_subparsers()
+#    parser_sso_config = parser_sso_sub.add_parser('config',
+#                        help='config aws sso.')
+#    parser_sso_config.set_defaults(func=_sso_config)
+#
+#    parser_sso_login = parser_sso_sub.add_parser('login',
+#                        help='ssm login into sso.')
+#    parser_sso_login.set_defaults(func=_sso_login)
+#
+#    parser_sso_switch = parser_sso_sub.add_parser('switch',
+#                        help='switch to another config.')
+#    parser_sso_switch.set_defaults(func=_sso_switch)
+#
+#    parser_sso_logout = parser_sso_sub.add_parser('logout',
+#                        help='logout aws sso.')
+#    parser_sso_logout.set_defaults(func=_sso_logout)
 
-    parser_sso_login = parser_sso_sub.add_parser('login',
-                        help='ssm login into sso.')
-    parser_sso_login.set_defaults(func=_sso_login)
+    parser_env = subparsers.add_parser('env')
+    parser_env_sub = parser_env.add_subparsers()
+    parser_env_get = parser_env_sub.add_parser('get')
+    parser_env_get.add_argument('get',
+                        help='get service env.')
+    parser_env_get.set_defaults(func=_env_get)
 
-    parser_sso_switch = parser_sso_sub.add_parser('switch',
-                        help='switch to another config.')
-    parser_sso_switch.set_defaults(func=_sso_switch)
-
-    parser_sso_logout = parser_sso_sub.add_parser('logout',
-                        help='logout aws sso.')
-    parser_sso_logout.set_defaults(func=_sso_logout)
+    parser_env_load = parser_env_sub.add_parser('load',
+                        help='load local file into env.')
+    parser_env_load.add_argument('load',
+                        help='lood local file into env.')
+    parser_env_load.set_defaults(func=_env_load)
 
 
     parser_tf = subparsers.add_parser('tf')
@@ -100,12 +130,17 @@ def main():
 
 
     parser_sm = subparsers.add_parser('sm')
-    parser_sm_sub = parser_sm.add_subparsers(help='sub func: search, create, update')
+    parser_sm_sub = parser_sm.add_subparsers(help='sub func: search, get, create, update')
     parser_sm_search = parser_sm_sub.add_parser('search')
     parser_sm_search.add_argument('search',
                         help='search secrets manager')
     parser_sm_search.set_defaults(func=_sm_search)
 
+    parser_sm_get = parser_sm_sub.add_parser('get')
+    parser_sm_get.add_argument('get',
+                        help='get secrets manager')
+    parser_sm_get.set_defaults(func=_sm_get)
+
     parser_sm_create = parser_sm_sub.add_parser('create')
     parser_sm_create.add_argument('create',
                         action='store_true',
@@ -122,15 +157,80 @@ def main():
                         help='delete secrets manager')
     parser_sm_delete.set_defaults(func=_sm_delete)
 
+    parser_image = subparsers.add_parser('image')
+    parser_image_sub = parser_image.add_subparsers(help='sub func: list, pull')
+    parser_image_list = parser_image_sub.add_parser('list')
+    parser_image_list.add_argument('list',
+                        help='list service images')
+    parser_image_list.set_defaults(func=_image_list)
+
+    parser_image_pull = parser_image_sub.add_parser('pull')
+    parser_image_pull.add_argument('pull',
+                        help='pull service image')
+    parser_image_pull.set_defaults(func=_image_pull)
+
+    parser_image_stat = parser_image_sub.add_parser('stat')
+    parser_image_stat.add_argument('stat',
+                        help='showing current image version in specific envs.')
+    parser_image_stat.set_defaults(func=_image_stat)
+
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
 
     if len(sys.argv)==1:
         parser.print_help()
+    if len(sys.argv) == 3 and sys.argv[1] == '-s':
+        _sso_switch(sys.argv[2])
+    if len(sys.argv) == 2 and sys.argv[1] in ('-l', '-c', '-s'):
+        if sys.argv[1] == '-l':
+            _sso_login()
+        if sys.argv[1] == '-c':
+            _sso_config()
+        if sys.argv[1] == '-s':
+            _sso_switch()
+        if sys.argv[1] == '-g':
+            _sso_logout()
     if len(sys.argv) == 2 and sys.argv[1] in subparsers.choices:
         print(subparsers.choices[sys.argv[1]].format_help())
 
-    # validate aws v2
+    if hasattr(args, 'func'):
+#        if args.func == _sso_config:
+#            _sso_config()
+#        if args.func == _sso_login:
+#            _sso_login()
+#        if args.func == _sso_switch:
+#            _sso_switch()
+#        if args.func == _sso_logout:
+#            _sso_logout()
+        if args.func == _env_get:
+            _env_get(args.get)
+        if args.func == _env_load:
+            _env_load(args.load)
+        if args.func == _tf_cmd:
+            _tf_cmd(args.cmd)
+        if args.func == _ec2_search:
+            _ec2_search(args.search)
+        if args.func == _ec2_login:
+            _ec2_login(args.login)
+        if args.func == _sm_search:
+            _sm_search(args.search)
+        if args.func == _sm_get:
+            _sm_get(args.get)
+        if args.func == _sm_create:
+            _sm_create()
+        if args.func == _sm_update:
+            _sm_update(args.update)
+        if args.func == _sm_delete:
+            _sm_delete(args.delete)
+        if args.func == _image_list:
+            _image_list(args.list)
+        if args.func == _image_pull:
+            _image_pull(args.pull)
+        if args.func == _image_stat:
+            _image_stat(args.stat)
+
+
+def _validate_awscli():
     try:
         aws_version = subprocess.run(['aws', '--version'], capture_output=True).stdout.decode('utf-8')
 
@@ -142,37 +242,23 @@ def main():
         print('\nAn error occured trying to find AWS CLI version. Do you have AWS CLI Version 2 installed?\n{e}')
         exit(1)
 
-    if hasattr(args, 'func'):
-        if args.func == _sso_config:
-            _sso_config()
-        if args.func == _sso_login:
-            _sso_login()
-        if args.func == _sso_switch:
-            _sso_switch()
-        if args.func == _sso_logout:
-            _sso_logout()
-        if args.func == _tf_cmd:
-            _tf_cmd(args.cmd)
-        if args.func == _ec2_search:
-            _ec2_search(args.search)
-        if args.func == _ec2_login:
-            _ec2_login(args.login)
-        if args.func == _sm_search:
-            _sm_search(args.search)
-        if args.func == _sm_create:
-            _sm_create()
-        if args.func == _sm_update:
-            _sm_update(args.update)
-        if args.func == _sm_delete:
-            _sm_delete(args.delete)
 
 def _sso_config():
+    _validate_awscli()
     _spawn_cli_for_config()
     sys.exit()
 
-def _sso_switch():
-    profile = _add_prefix(_select_profile())
-    profile_opts = _get_aws_profile(profile)
+def _sso_switch(env=''):
+    _validate_awscli()
+    if not env:
+        profile = _add_prefix(_select_profile())
+    else:
+        profile = 'profile ' + env
+    try:
+        profile_opts = _get_aws_profile(profile)
+    except:
+        print('The profile: {} is not found.'.format(env))
+        sys.exit(1)
     profile_name = profile.split(' ')[1]
     source_value = '''
 
@@ -200,12 +286,14 @@ source ~/.aws_env
     _set_profile_credentials(profile)
 
 def _sso_logout():
+    _validate_awscli()
     select_one = True
     profile = _add_prefix(_select_profile(select_one))
     _spawn_cli_for_logout(profile)
 
 
 def _sso_login():
+    _validate_awscli()
     select_one = True
     profile = _add_prefix(_select_profile(select_one))
     _spawn_cli_for_auth(profile)
@@ -523,6 +611,15 @@ def _sm_search(keyword):
     ret = client.get_secret_value(SecretId=answer['name'])
     print(ret['SecretString'])
 
+def _sm_get(secret):
+    client = boto3.client('secretsmanager', region_name=CURRENT_REGION)
+    try:
+        ret = client.get_secret_value(SecretId=secret)
+        print(ret['SecretString'])
+    except:
+        print('Get secrets error.')
+        sys.exit(1)
+
 def _sm_create():
     client = boto3.client('secretsmanager', region_name=CURRENT_REGION)
     sm_name = input('Please enter the secrets manager:\n')
@@ -565,6 +662,172 @@ def _sm_delete(keyword):
         print('The sm: {} will not be deleted'.format(keyword))
         sys.exit()
 
+def _env_get(service):
+
+    def GetPathToken(path, Token):
+
+        try:
+            client = boto3.client('ssm')
+            rc = client.get_parameters_by_path(
+                Path=path,
+                Recursive=True,
+                WithDecryption=True,
+                MaxResults=10,
+                NextToken=Token
+            )
+            rs = rc['Parameters']
+        except client.exceptions.ClientError as e:
+            print("ERROR: GetPathToken %s=%s" % (path, e))
+            sys.exit(1)
+
+        token = ""
+
+        try:
+            token = rc['NextToken']
+        except:
+            token = ""
+
+        return rs, token
+
+    def GetPath(path):
+        try:
+            client = boto3.client('ssm')
+            rc = client.get_parameters_by_path(
+                Path=path,
+                Recursive=True,
+                WithDecryption=True
+            )
+            rs = rc['Parameters']
+        except client.exceptions.ClientError as e:
+            print("ERROR: GetPath %s=%s" % (path, e))
+            sys.exit(1)
+
+        try:
+            token = rc['NextToken']
+        except:
+            token = ""
+
+        return rs, token
+
+    def GetData(path):
+        data = []
+        rs, token = GetPath(path)
+        for row in rs:
+            data.append(dict(Name=row['Name'], Value=row['Value']))
+
+        while token:
+            rs2, token = GetPathToken(path, token)
+            for row in rs2:
+                data.append(dict(Name=row['Name'], Value=row['Value']))
+
+        return data
+
+    path = '/configs/services/{}/envs/default'.format(service)
+    env_data = GetData(path)
+    source_value = ''
+    print('')
+    for i in env_data:
+        i_iter = '{}={}'.format(i['Name'].replace(path + '/', ''), i['Value'])
+        source_value += '\n' + i_iter
+        print(i_iter)
+
+    os.system('echo "{}" > ~/.{}_env'.format(source_value, service))
+    print('')
+    print('The env is also exported to ~/.{}_env'.format(service))
+    print('')
+
+def _env_load(service):
+
+    def Info(file):
+        tag=""
+        info=[]
+
+        with open(file, "r") as f:
+            try:
+                data=yaml.load(f,Loader=yaml.FullLoader)
+                tag=data['ssm']
+            except yaml.YAMLError as exc:
+                print(exc)
+
+        try:
+            for key,value in data.items():
+                if key == "ssm":
+                    continue
+                if key == "global" or key == os.environ['AWS_ENVS']:
+                    for key2, value2 in data[key].items():
+                        param2="%s/%s" % (tag,key2)
+                        info.append(dict(Param=param2,Key=key2,Value=value2))
+        except yaml.YAMLError as exc:
+            print(exc)
+
+        return tag, info
+
+    tag, info = Info(service)
+    client = boto3.client('ssm')
+    print("")
+    print("Load:", tag)
+    print("")
+    for item in info:
+        value=item['Value']
+        tier = "Advanced" if len(value) > 4096 else "Standard"
+        print(item['Param'], "=", value)
+
+        try:
+            response = client.put_parameter(
+                Name=item['Param'],
+                Value=value,
+                Type='String',
+                Overwrite=True,
+                Tier=tier
+            )
+        except Exception as e:
+            print("ERROR: PutConfig %s=%s" % (name, e))
+            sys.exit(1)
+        except:
+            print("ERROR: PutConfig %s" % (name))
+            sys.exit(1)
+
+    print("")
+
+def _image_list(service):
+    client = boto3.client('ecr')
+    try:
+        ret = client.list_images(
+            registryId=AWS_OPS1_ACCOUNT,
+            repositoryName=service,
+            filter={
+                'tagStatus': 'TAGGED'
+            }
+        )
+    except:
+        print('Service {} ECR is not exist or you donot have permission to list ECR.'.format(service))
+        sys.exit(1)
+
+    _ret_list = ret['imageIds']
+    _next = ret.get('NextToken')
+    while _next:
+        ret_tmp = client.list_images(MaxResults=100, NextToken=_next)
+        _ret_list.extend(ret_tmp['imageTag'])
+        _next = ret_tmp.get('NextToken')
+    ret_list = []
+    for i in _ret_list:
+        ret_list.append(i['imageTag'])
+    for j in ret_list:
+        print('{}.dkr.ecr.{}.amazonaws.com/{}:{}'.format(AWS_OPS1_ACCOUNT, AWS_OPS1_REGION, service, j))
+
+def _image_pull(image):
+    subprocess.run('aws ecr get-login-password --region {1} | docker login --username AWS --password-stdin {0}.dkr.ecr.{1}.amazonaws.com'.format(AWS_OPS1_ACCOUNT, AWS_OPS1_REGION),
+        shell=True,
+        stderr=sys.stderr,
+        stdout=sys.stdout,
+        check=True)
+    subprocess.run(['docker', 'pull', image],
+        stderr=sys.stderr,
+        stdout=sys.stdout,
+        check=True)
+
+def _image_stat(service):
+    pass
 
 if __name__ == "__main__":
     main()
